require 'rest_client'
require 'json'

module Fosbury
  class Client

    @endpoint

    def initialize
      @endpoint = Fosbury.endpoint || "https://app.fosbury.co/api/v1/"
    end

    def get_templates
      get "#{@endpoint}templates"
    end

    def get_campaigns
      get "#{@endpoint}campaigns"
    end

    def get_passes
      get "#{@endpoint}passes"
    end

    def get_template id
      get "#{@endpoint}templates/#{id}"
    end

    def get_campaign id
      get "#{@endpoint}campaigns/#{id}"
    end

    def get_pass id
      get "#{@endpoint}passes/#{id}"
    end

    def create_template name, style, options={}
      options[:name] = name
      options[:style] = style

      post "#{@endpoint}templates", options
    end

    def create_campaign template_id, options={}
      options[:template_id] = template_id
      post "#{@endpoint}campaigns", options
    end

    def create_pass campaign_id, options={}
      raise "Please provide a campaign_id" if campaign_id.nil?
      options[:campaign_id] = campaign_id
      post "#{@endpoint}passes", options
    end

    def update_template template_id, options={}
      raise "Please provide a template_id" if template_id.nil?
      put "#{@endpoint}tempates/#{template_id}", options
    end

    def update_campaign campaign_id, options={}
      raise "Please provide a campaign_id" if campaign_id.nil?
      put "#{@endpoint}campaigns/#{campaign_id}", options
    end

    def update_pass pass_id, options={}
      put "#{@endpoint}passes/#{pass_id}", options
    end

    def delete_template teplate_id
      delete "#{@endpoint}tempates/#{template_id}"
    end

    def delete_campaign campaign_id
      delete "#{@endpoint}campaigns/#{campaign_id}"
    end

    def delete_pass pass_id
      delete "#{@endpoint}passes/#{pass_id}"
    end

    def push_pass pass_id, options={}
      put "#{@endpoint}passes/#{pass_id}/push", options
    end

    def archive_pass pass_id
      post "#{@endpoint}passes/#{pass_id}/archive"
    end

    def redeem_pass pass_id
      post "#{@endpoint}passes/#{pass_id}/redeem"
    end

    def distribute_campaign campaign_id
      raise "Please provide a campaign_id" if campaign_id.nil?
      post "#{@endpoint}campaigns/#{campaign_id}/distribute"
    end

    def get_campaign_locations campaign_id
      raise "Please provide a campaign_id" if campaign_id.nil?
      get "#{@endpoint}/campaigns/#{campaign_id}/locations"
    end

    def create_campaign_location campaign_id, lat, long, name, relevant_text, address=''
      raise "Please provide a campaign_id" if campaign_id.nil?
      options = { campaign_id: campaign_id, lat: lat, long: long, name: name, relevant_text: relevant_text, address: address}
      post "#{@endpoint}campaigns/#{campaign_id}/locations", options
    end

    def delete_campaign_location campaign_id, location_id
      raise "Please provide a campaign_id" if campaign_id.nil?
      raise "Please provide a location_id" if location_id.nil?
      delete "#{@endpoint}/campaigns/#{campaign_id}/locations/#{location_id}"
    end

    def get_campaign_backfields campaign_id
      raise "Please provide a campaign_id" if campaign_id.nil?
      get "#{@endpoint}/campaigns/#{campaign_id}/backfields"
    end

    def create_campaign_backfield campaign_id, title, description
      raise "Please provide a campaign_id" if campaign_id.nil?
      options = { title: title, description: description }
      post "#{@endpoint}campaigns/#{campaign_id}/backfields", options
    end

    def delete_campaign_backfield campaign_id, backfield_id
      raise "Please provide a campaign_id" if campaign_id.nil?
      raise "Please provide a backfield_id" if backfield_id.nil?
      delete "#{@endpoint}/campaigns/#{campaign_id}/backfields/#{backfield_id}"
    end

    def get_campaign_beacons campaign_id
      raise "Please provide a campaign_id" if campaign_id.nil?
      get "#{@endpoint}/campaigns/#{campaign_id}/ibeacons"
    end

    def create_campaign_beacon campaign_id, proximity_uuid, relevant_text, major=nil, minor=nil
      raise "Please provide a campaign_id" if campaign_id.nil?
      options = { campaign_id: campaign_id, proximity_uuid: proximity_uuid, relevant_text: relevant_text}
      options[:major] = major if !major.nil?
      options[:minor] = minor if !minor.nil?
      post "#{@endpoint}campaigns/#{campaign_id}/ibeacons", options
    end

    def delete_campaign_beacon campaign_id, beacon_id
      raise "Please provide a campaign_id" if campaign_id.nil?
      raise "Please provide a beacon_id" if beacon_id.nil?
      delete "#{@endpoint}/campaigns/#{campaign_id}/ibeacons/#{beacon_id}"
    end

    def get_programs
      get "#{@endpoint}programs"
    end

    def get_program id
      get "#{@endpoint}programs/#{id}"
    end

    def update_program program_id, options={}
      raise "Please provide a program_id" if program_id.nil?
      put "#{@endpoint}programs/#{campaign_id}", options
    end
    
    def push_program program_id, options={}
      raise "Please provide a program_id" if program_id.nil?
      post "#{@endpoint}programs/#{program_id}/push_message", options
    end

    def get_program_members program_id
      raise "Please provide a program_id" if program_id.nil?
      get "#{@endpoint}programs/#{program_id}/members"
    end

    def get_program_member program_id, member_id
      raise "Please provide a program_id" if program_id.nil?
      raise "Please provide a member_id" if member_id.nil?
      get "#{@endpoint}programs/#{program_id}/members/#{member_id}"
    end

    def create_program_member program_id, customer_number, first_name, last_name
      raise "Please provide a program_id" if program_id.nil?
      raise "Please provide a customer_number" if customer_number.nil?
      raise "Please provide a first_name" if first_name.nil?
      raise "Please provide a last_name" if last_name.nil?

      options = { customer_number: customer_number, first_name: first_name, last_name: last_name}

      post "#{@endpoint}programs/#{program_id}/members", options
    end

    def update_program_member program_id, member_id, customer_number, first_name, last_name, email=nil
      raise "Please provide a program_id" if program_id.nil?
      raise "Please provide a member_id" if member_id.nil?
      raise "Please provide a customer_number" if customer_number.nil?
      raise "Please provide a first_name" if first_name.nil?
      raise "Please provide a last_name" if last_name.nil?

      options = { customer_number: customer_number, first_name: first_name, last_name: last_name, email: email}

      put "#{@endpoint}programs/#{program_id}/members/#{member_id}", options
    end

    def delete_program_member program_id, member_id
      raise "Please provide a program_id" if program_id.nil?
      raise "Please provide a member_id" if member_id.nil?
      delete "#{@endpoint}programs/#{program_id}/members/#{member_id}"
    end

    def get_program_locations program_id
      raise "Please provide a program_id" if program_id.nil?
      get "#{@endpoint}/programs/#{program_id}/locations"
    end

    def create_program_location program_id, lat, long, name, address=''
      raise "Please provide a program_id" if program_id.nil?
      options = { program_id: program_id, lat: lat, long: long, name: name, address: address }
      post "#{@endpoint}programs/#{program_id}/locations", options
    end

    def delete_program_location program_id, location_id
      raise "Please provide a program_id" if program_id.nil?
      raise "Please provide a location_id" if location_id.nil?
      delete "#{@endpoint}/programs/#{program_id}/locations/#{location_id}"
    end

    def get_program_backfields program_id
      raise "Please provide a program_id" if program_id.nil?
      get "#{@endpoint}/programs/#{program_id}/backfields"
    end

    def create_program_backfield program_id, title, description
      raise "Please provide a program_id" if program_id.nil?
      options = { title: title, description: description }
      post "#{@endpoint}programs/#{program_id}/backfields", options
    end

    def delete_program_backfield program_id, backfield_id
      raise "Please provide a program_id" if program_id.nil?
      raise "Please provide a backfield_id" if backfield_id.nil?
      delete "#{@endpoint}/programs/#{program_id}/backfields/#{backfield_id}"
    end

    def get_program_beacons program_id
      raise "Please provide a program_id" if program_id.nil?
      get "#{@endpoint}/programs/#{program_id}/ibeacons"
    end

    def create_program_beacon program_id, proximity_uuid, relevant_text, major=nil, minor=nil
      raise "Please provide a program_id" if program_id.nil?
      options = { program_id: program_id, proximity_uuid: proximity_uuid, relevant_text: relevant_text}
      options[:major] = major if !major.nil?
      options[:minor] = minor if !minor.nil?
      post "#{@endpoint}programs/#{program_id}/ibeacons", options
    end

    def delete_program_beacon program_id, beacon_id
      raise "Please provide a program_id" if program_id.nil?
      raise "Please provide a beacon_id" if beacon_id.nil?
      delete "#{@endpoint}/programs/#{program_id}/ibeacons/#{beacon_id}"
    end

    private

    def get url
      response = RestClient.get url, get_headers
      parse(response.to_s)
    end

    def post url, payload=''
      response = RestClient.post url, payload, get_headers
      parse(response.to_s)
    end

    def put url, payload=''
      response = RestClient.put url, payload, get_headers
      parse(response.to_s)
    end

    def delete url
      response = RestClient.delete url, get_headers 
      parse(response)
    end

    def parse json_str
      return nil if json_str.empty?
      JSON.parse(json_str)
    end

    def get_headers
      raise "No authentication token provided. Use Fosbury.authentication_token = 'yourtoken' first" if Fosbury.authentication_token.nil?
      {'X-Fosbury-Token' => Fosbury.authentication_token}
    end    
  end
end